#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>


// ---- LCD Definitions ----
#define LCD_DATA_PORT          PORTB
#define LCD_DATA_PIN          PINB
#define LCD_DATA_DDR          DDRB
#define LCD_CONTROL_PORT       PORTC
#define LCD_CONTROL_PIN       PINC
#define LCD_CONTROL_DDR       DDRC
#define LCD_RS             2
#define LCD_RW             1
#define LCD_ENABLE          0
#define LCD_CS1_PIN          4
#define LCD_CS2_PIN          3
#define LCD_RESET         5

#define DISPLAY_ON         0b00111111
#define DISPLAY_OFF         0b00111110
#define DISPLAY_STATUS_BUSY      0b10000000
#define DISPLAY_STATUS_ON_OFF      0b00100000
#define DISPLAY_STATUS_RESET      0b00010000
#define DISPLAY_LINE         0b11000000
#define ADDRESS_X         0b01000000
#define ADDRESS_Y         0b10111000

#define SET_CS1()          (LCD_CONTROL_PORT |= (1<<LCD_CS1_PIN))
#define CLEAR_CS1()          (LCD_CONTROL_PORT &= ~(1<<LCD_CS1_PIN))
#define SET_CS2()          (LCD_CONTROL_PORT |= (1<<LCD_CS2_PIN))
#define CLEAR_CS2()          (LCD_CONTROL_PORT &= ~(1<<LCD_CS2_PIN))
#define SET_ENABLE()         (LCD_CONTROL_PORT |= (1<<LCD_ENABLE))
#define CLEAR_ENABLE()         (LCD_CONTROL_PORT &= ~(1<<LCD_ENABLE))
#define SET_RW()          (LCD_CONTROL_PORT |= (1<<LCD_RW))
#define CLEAR_RW()          (LCD_CONTROL_PORT &= ~(1<<LCD_RW))
#define SET_RS()          (LCD_CONTROL_PORT |= (1<<LCD_RS))
#define CLEAR_RS()          (LCD_CONTROL_PORT &= ~(1<<LCD_RS))
// ------------


const char smallFont[] =
{
     0x00, 0x00, 0x00, 0x00, 0x00,   // sp
     0x00, 0x00, 0x5f, 0x00, 0x00,    // !
     0x00, 0x07, 0x00, 0x07, 0x00,   // "
     0x14, 0x7f, 0x14, 0x7f, 0x14,   // #
     0x24, 0x2a, 0x7f, 0x2a, 0x12,   // $
     0x23, 0x13, 0x08, 0x64, 0x62,   // %
     0x36, 0x49, 0x55, 0x22, 0x50,   // &
     0x00, 0x05, 0x03, 0x00, 0x00,   // '
     0x00, 0x1c, 0x22, 0x41, 0x00,   // (
     0x00, 0x41, 0x22, 0x1c, 0x00,   // )
     0x14, 0x08, 0x3E, 0x08, 0x14,   // *
     0x08, 0x08, 0x3E, 0x08, 0x08,   // +
     0x00, 0x00, 0x50, 0x30, 0x00,   // ,
     0x10, 0x10, 0x10, 0x10, 0x10,   // -
     0x00, 0x60, 0x60, 0x00, 0x00,   // .
     0x20, 0x10, 0x08, 0x04, 0x02,   // /
     0x3E, 0x51, 0x49, 0x45, 0x3E,   // 0
     0x00, 0x42, 0x7F, 0x40, 0x00,   // 1
     0x42, 0x61, 0x51, 0x49, 0x46,   // 2
     0x21, 0x41, 0x45, 0x4B, 0x31,   // 3
     0x18, 0x14, 0x12, 0x7F, 0x10,   // 4
     0x27, 0x45, 0x45, 0x45, 0x39,   // 5
     0x3C, 0x4A, 0x49, 0x49, 0x30,   // 6
     0x01, 0x71, 0x09, 0x05, 0x03,   // 7
     0x36, 0x49, 0x49, 0x49, 0x36,   // 8
     0x06, 0x49, 0x49, 0x29, 0x1E,   // 9
     0x00, 0x36, 0x36, 0x00, 0x00,   // :
     0x00, 0x56, 0x36, 0x00, 0x00,   // ;
     0x08, 0x14, 0x22, 0x41, 0x00,   // <
     0x14, 0x14, 0x14, 0x14, 0x14,   // =
     0x00, 0x41, 0x22, 0x14, 0x08,   // >
     0x02, 0x01, 0x51, 0x09, 0x06,   // ?
     0x32, 0x49, 0x59, 0x51, 0x3E,   // @
     0x7E, 0x11, 0x11, 0x11, 0x7E,   // A
     0x7F, 0x49, 0x49, 0x49, 0x36,   // B
     0x3E, 0x41, 0x41, 0x41, 0x22,   // C
     0x7F, 0x41, 0x41, 0x22, 0x1C,   // D
     0x7F, 0x49, 0x49, 0x49, 0x41,   // E
     0x7F, 0x09, 0x09, 0x09, 0x00,   // F
     0x3E, 0x41, 0x49, 0x49, 0x7A,   // G
     0x7F, 0x08, 0x08, 0x08, 0x7F,   // H
     0x00, 0x41, 0x7F, 0x41, 0x00,   // I
     0x20, 0x40, 0x41, 0x3F, 0x01,   // J
     0x7F, 0x08, 0x14, 0x22, 0x41,   // K
     0x7F, 0x40, 0x40, 0x40, 0x40,   // L
     0x7F, 0x02, 0x0C, 0x02, 0x7F,   // M
     0x7F, 0x04, 0x08, 0x10, 0x7F,   // N
     0x3E, 0x41, 0x41, 0x41, 0x3E,   // O
     0x7F, 0x09, 0x09, 0x09, 0x06,   // P
     0x3E, 0x41, 0x51, 0x21, 0x5E,   // Q
     0x7F, 0x09, 0x19, 0x29, 0x46,   // R
     0x46, 0x49, 0x49, 0x49, 0x31,   // S
     0x01, 0x01, 0x7F, 0x01, 0x01,   // T
     0x3F, 0x40, 0x40, 0x40, 0x3F,   // U
     0x1F, 0x20, 0x40, 0x20, 0x1F,   // V
     0x3F, 0x40, 0x38, 0x40, 0x3F,   // W
     0x63, 0x14, 0x08, 0x14, 0x63,   // X
     0x07, 0x08, 0x70, 0x08, 0x07,   // Y
     0x61, 0x51, 0x49, 0x45, 0x43,   // Z
     0x00, 0x7F, 0x41, 0x41, 0x00,   // [
     0x55, 0x2A, 0x55, 0x2A, 0x55,   // 55
     0x00, 0x41, 0x41, 0x7F, 0x00,   // ]
     0x04, 0x02, 0x01, 0x02, 0x04,   // ^
     0x40, 0x40, 0x40, 0x40, 0x40,   // _
     0x00, 0x01, 0x02, 0x04, 0x00,   // '
     0x20, 0x54, 0x54, 0x54, 0x78,   // a
     0x7F, 0x48, 0x44, 0x44, 0x38,    // b
     0x38, 0x44, 0x44, 0x44, 0x20,   // c
     0x38, 0x44, 0x44, 0x48, 0x7F,   // d
     0x38, 0x54, 0x54, 0x54, 0x18,   // e
     0x08, 0x7E, 0x09, 0x01, 0x02,   // f
     0x0C, 0x52, 0x52, 0x52, 0x3E,   // g
     0x7F, 0x08, 0x04, 0x04, 0x78,   // h
     0x00, 0x44, 0x7D, 0x40, 0x00,   // i
     0x20, 0x40, 0x44, 0x3D, 0x00,   // j
     0x7F, 0x10, 0x28, 0x44, 0x00,   // k
     0x00, 0x41, 0x7F, 0x40, 0x00,   // l
     0x7C, 0x04, 0x18, 0x04, 0x78,   // m
     0x7C, 0x08, 0x04, 0x04, 0x78,   // n
     0x38, 0x44, 0x44, 0x44, 0x38,   // o
     0x7C, 0x14, 0x14, 0x14, 0x08,   // p
     0x08, 0x14, 0x14, 0x18, 0x7C,   // q
     0x7C, 0x08, 0x04, 0x04, 0x08,   // r
     0x48, 0x54, 0x54, 0x54, 0x20,   // s
     0x04, 0x3F, 0x44, 0x40, 0x20,   // t
     0x3C, 0x40, 0x40, 0x20, 0x7C,   // u
     0x1C, 0x20, 0x40, 0x20, 0x1C,   // v
     0x3C, 0x40, 0x30, 0x40, 0x3C,   // w
     0x44, 0x28, 0x10, 0x28, 0x44,   // x
     0x0C, 0x50, 0x50, 0x50, 0x3C,   // y
     0x44, 0x64, 0x54, 0x4C, 0x44,   // z
	 0x00, 0x06, 0x09, 0x09, 0x06    // 
};

/*
Info - How to use the commands.

1. LCD_Fill_Screen();
This command will fill the screen with pixels.

2. LCD_Clear_Screen();
This command will clear the screen of pixels.

3. LCD_Draw_Dot(x, y);
This command will draw a dot at the specified x, y location.

4. LCD_Clear_Dot(x, y);
This command will clear a dot at the specified x, y location.

5. LCD_Draw_Line(x1, y1, x2, y2, clr);
This command will draw a line from the specified cordinates (x1, y1) to the specified cordinates (x2, y2).
If clr is 1 it will clear all pixels of the line and if it is 0 it will draw a line.

6. LCD_Draw_Rect(x1, y1, x2, y2, fill, clr);
This command will draw a box from the specified cordinates (x1, y1) to the specified cordinates (x2, y2).
if fill is 1 it will fill the box and if it is 0 it will just make the outline of a box.
If clr is 1 it will clear all pixels of the box and if it is 0 it will draw the box.

7. LCD_Draw_Circle(x, y, R);
This command will draw a circle at the specified cordinates (x, y) with the radius 'R'.

8. LCD_Write_Char(x, y, 'char');
This command will draw a character ('char') at the specified cordinates (x, y), the x and y cordinates determins the character's top-left pixel.

9. LCD_Write_String(x, y, "string");
This command will draw a string ("string") at the specified cordinates (x, y), the x and y cordinates determins the string's top-left pixel.
*/



//#define F_CPU               8000000

void LCD_Init(void);
void LCD_Wait(void);
void LCD_Fill_Screen(void);
void LCD_Clear_Screen(void);
void LCD_Send_Cmd(unsigned char);
void LCD_Send_Data(unsigned char);
void LCD_Set_Pos(unsigned char, unsigned char);
void LCD_Draw_Dot(unsigned char, unsigned char);
void LCD_Clear_Dot(unsigned char, unsigned char);
void LCD_Draw_Line(unsigned char, unsigned char, unsigned char, unsigned char, unsigned char);
void LCD_Draw_Rect(unsigned char, unsigned char, unsigned char, unsigned char, unsigned char, unsigned char);
void LCD_Draw_Circle(unsigned char, unsigned char, unsigned char);
void LCD_Write_Char(unsigned char, unsigned char, char);
void LCD_Write_String(unsigned char, unsigned char, char *);
